#!/usr/bin/perl

##############################################################################
#  Copyright (C) 2007, 2008 Paulo Cesar Pereira de Andrade.  All Rights Reserved.
#
#  This is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This software is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  Authors:
#	Paulo Cesar Pereira de Andrade <pcpa@mandriva.com.br>
##############################################################################

use Cwd;
use File::Basename;
use Getopt::Std;

use strict;


##############################################################################
# Check command line options
my %options;
my $command = "";
my $arguments = "";

# File name to know finished a build with no erros
my $build_ok = "xorg-build.ok";

# Suffix for package names
my $package_suffix = "-snapshot";
my $package_destdir = "/opt/xorg-snapshot";

my $rpm;


##############################################################################
# This hash table is used to allow creating aliases to commads
my %commands =
(
 "up"		=> "update",
 "update"	=> "update",
 "co"		=> "checkout",
 "checkout"	=> "checkout",
 "build"	=> "build",
 "checktars"	=> "checktars",
 "checktarballs"=> "checktars",
 "sh"		=> "shell",
 "shell"	=> "shell",

 "rpm"		=> "rpm",
 );


##############################################################################
# Check -<letter> options and script command
Getopt::Std::getopts("n:lcdfgvhp:?", \%options);
$command = shift(@ARGV);
$command = $commands{$command};
if ($command eq "shell") {
    $arguments = join(" ", @ARGV);
}
elsif (@ARGV) {
    # just to cause an error
    $command = "";
}
if (not $commands{$command} or $options{'h'} or $options{'?'}) {
    print(STDERR "Usage:\n",
	  File::Basename::basename($0), " [options ...] command\n",
	  "Options:\n",
	  " -c      Start over update instead of from last failed repository.\n",
	  " -d      Makes a debug build.\n",
	  " -g      Generate dependency files (uses strace).\n",
	  " -f      Force resubmission of a package.\n",
	  " -v      Verbose mode.\n",
	  " -n name Process only modules with given name.\n",
	  " -p name Process only modules with given package name.\n",
	  " -h, -?  Prints this message and quits.\n",
	  "Commands:\n",
	  " update         Updates source checkouts.\n",
	  " checkout       Create build directory.\n",
	  " build          Rebuild module.\n",
	  " checktarballs  Checks if package tarball is available upstream.\n",
	  " shell command  Executes <command> in checkout/build directory.\n",

	  " rpm            Manages rpm snapshots\n",
	  "\n");
    exit(0);
}

if ($command eq "rpm") {
    $rpm = 1;
}

# For the moment, check it only once. Required for some fallback default
# value options.
die ("\$ENV{HOME} not set. Cannot continue.") unless ($ENV{HOME} ne "");

# Maybe -W options could be handled differently, but a debug build most
# certainly wants at least these...
$ENV{CFLAGS} = "-O0 -g3 "	.
"-Wall "			.
"-Wbad-function-cast "		.
"-Wdeclaration-after-statement ".
"-Wmissing-prototypes "		.
"-Wmissing-declarations "	.
"-Wnested-externs "		.
"-fno-strict-aliasing "		.
"-Wold-style-definition "	.
"-Wpointer-arith "		.
"-Wstrict-prototypes " if ($options{'d'});

# For error messages
$ENV{LC_ALL} = "C";


##############################################################################
# User configuration
sub git_config {
    my ($option) = @_;

    $option = `git-config --global --get $option`;
    $option =~ s/^\s+//;
    $option =~ s/\s+$//;

    return $option;
};

my $xorg_git_protocol = git_config("xorg-git.protocol");
$xorg_git_protocol = "git"
    unless ($xorg_git_protocol ne "");

my $xorg_git_repository = git_config("xorg-git.repository");
$xorg_git_repository = "anongit.freedesktop.org/git"
    unless ($xorg_git_repository ne "");

my $git_user_name = git_config("user.name");
die("* Please run:\n",
    "\tgit-config --global user.name \"Your user name\"\n",
    "before proceeding.")
    unless ($git_user_name ne "");

my $git_user_email = git_config("user.email");
die("* Please run:\n",
    "\tgit-config --global user.email you\@yourdomain.example.com\n",
    "before proceeding.")
    unless ($git_user_email ne "");

# Example: lib or lib64
my $xorg_git_lib_name = git_config("xorg-git.lib-name");
if ($xorg_git_lib_name eq "") {
    if ($options{'v'}) {
	print(STDERR "* Please consider running:\n",
	      "\tgit-config --global xorg-git.lib-name name-of-library-path\n");
    }
    # Try to guess proper default
    $xorg_git_lib_name = `uname -m`;
    if ($rpm) {
	$xorg_git_lib_name = "lib";
    }
    else {
	$xorg_git_lib_name = $xorg_git_lib_name =~ m/64/ ? "lib64" : "lib";
    }
}

# Example: sudo or empty string
#### Was broken "Tue Jun 10 2008" for mesa because it would rebuild some
#### up to date libraries in make install...
my $xorg_git_sudo_command = git_config("xorg-git.sudo-command");
if ($xorg_git_sudo_command eq "") {
    if ($options{'v'}) {
	print(STDERR "* Please consider running:\n",
	      "\tgit-config --global xorg-git.sudo-command \"name-of-sudo-command-or-empty-string\"\n");
    }
    # Default to empty string, this should also help in catching attempting
    # to install files outside destdir
    $xorg_git_sudo_command = "";
}

# Example: /home/paulo/anongit.freedesktop.org
my $xorg_git_checkout_dir = git_config("xorg-git.checkout-dir");
if ($xorg_git_checkout_dir eq "") {
    if ($options{'v'}) {
	print(STDERR "* Please consider running:\n",
	      "\tgit-config --global xorg-git.checkout-dir /path/to/git/checkout/dir\n");
    }
    $xorg_git_checkout_dir = "$ENV{HOME}/anongit.freedesktop.org";
}

# Example: /home/paulo/anongit.freedesktop.org/build
my $xorg_git_build_dir = git_config("xorg-git.build-dir");
if ($xorg_git_build_dir eq "") {
    if ($options{'v'}) {
	print(STDERR "* Please consider running:\n",
	      "\tgit-config --global xorg-git.build-dir /path/to/build/dir\n");
    }
    $xorg_git_build_dir = "$ENV{HOME}/anongit.freedesktop.org/build";
}

# Example: /usr/local/xorg
my $xorg_git_dest_dir = git_config("xorg-git.dest-dir");
if ($xorg_git_dest_dir eq "") {
    if ($options{'v'}) {
	print(STDERR "* Please consider running:\n",
	      "\tgit-config --global xorg-git.dest-dir /path/to/install/binaries\n");
    }
    $xorg_git_dest_dir = "/usr/local/xorg";
}

# Example: /usr/local/xorg/deps
my $xorg_git_deps_dir = git_config("xorg-git.deps-dir");
if ($xorg_git_deps_dir eq "") {
    if ($options{'v'}) {
	print(STDERR "* Please consider running:\n",
	      "\tgit-config --global xorg-git.deps-dir /path/to/install/dependencies\n");
    }
    $xorg_git_deps_dir = "$ENV{HOME}/anongit.freedesktop.org/deps";
}

# Example: /home/paulo/rpm
my $xorg_git_rpm_dir = git_config("xorg-git.rpm-dir");
if ($xorg_git_rpm_dir eq "") {
    if ($rpm and $options{'v'}) {
	print(STDERR "* Please consider running:\n",
	      "\tgit-config --global xorg-git.rpm-dir /path/to/build/and/checkout/rpms\n");
    }
    $xorg_git_rpm_dir = "$ENV{HOME}/rpm";
}

if ($options{'v'}) {
    print(STDERR "Using options:\n",
	  "  xorg-git.protocol     = $xorg_git_protocol\n",
	  "  xorg-git.repository   = $xorg_git_repository\n",
	  "  git_user_name         = $git_user_name\n",
	  "  git_user_email        = $git_user_email\n",
	  "  xorg-git.checkout-dir = $xorg_git_checkout_dir\n",
	  "  xorg-git.build-dir    = $xorg_git_build_dir\n",
	  "  xorg-git.dest-dir     = $xorg_git_dest_dir\n",
	  "  xorg-git.deps-dir     = $xorg_git_deps_dir\n",
	  "  xorg-git.lib-name     = $xorg_git_lib_name\n",
	  "  xorg-git.sudo-command = $xorg_git_sudo_command\n",
	  ($rpm ? "  xorg-git.rpm-dir      = $xorg_git_rpm_dir\n" : ""),
	  "\n");
}

# Need these to exist
die("$xorg_git_checkout_dir does not exist.")
    unless (-d $xorg_git_checkout_dir);
die("$xorg_git_build_dir does not exist.")
    unless (-d $xorg_git_build_dir);
die("$xorg_git_dest_dir does not exist.")
    unless (-d $xorg_git_dest_dir);
die("$xorg_git_deps_dir does not exist.")
    unless (-d $xorg_git_deps_dir);
if ($rpm) {
    die("$xorg_git_rpm_dir does not exist.")
	unless (-d $xorg_git_rpm_dir);
}

my $xorg_trace_pl = `which xorg-trace.pl`;
chomp($xorg_trace_pl);
if ($options{'g'}) {
    $xorg_trace_pl = "./xorg-trace.pl" unless (-f $xorg_trace_pl);
    die ("$xorg_trace_pl: ($!)") unless (-f $xorg_trace_pl);
    $xorg_trace_pl = "/usr/bin/perl $xorg_trace_pl";
}

my $auto_cache_file = "$xorg_git_build_dir/.cache";
my $git_last_update = "$xorg_git_checkout_dir/xorg-update.tmp";

my $pkgconfig_path =  "$xorg_git_dest_dir/$xorg_git_lib_name/pkgconfig";
$ENV{PKG_CONFIG_PATH} = "$pkgconfig_path:$ENV{PKG_CONFIG_PATH}";
$ENV{LD_LIBRARY_PATH} = "$xorg_git_dest_dir/$xorg_git_lib_name:$ENV{LD_LIBRARY_PATH}";
$ENV{ACLOCAL_LOCALDIR} = "$xorg_git_dest_dir/share/aclocal";
$ENV{ACLOCAL} = "aclocal -I $ENV{ACLOCAL_LOCALDIR}";

# Don't fail so early on first run :-)
if (not -d $ENV{ACLOCAL_LOCALDIR}) {
    x_system("$xorg_git_sudo_command mkdir -p $ENV{ACLOCAL_LOCALDIR}");
}
# For some reason, some pkgconfig files were not being installed, and
# it was messing with the dependencies or reading "system" files when
# installing in a test directory.
if (not -d $pkgconfig_path) {
    x_system("$xorg_git_sudo_command mkdir -p $pkgconfig_path");
}

$ENV{PATH} = "$xorg_git_dest_dir/bin:$ENV{PATH}";


##############################################################################
# List of modules. Note that it is stored as a list also to allow proper
# specification of order of packages that must be updated first.
# Fields are:
#	name		-> Name of module (also directory to append to path)
#	path		-> Pathname in git repository
#	configure	-> Extra configure/autogen.sh parameters
#	version		-> Module version
#	commit		-> If defined, build this commit/tag/branch
#			   use with care...
#	tarball		-> URL for tarball of module. This usually
#			   is the official tarball of the module
#	skip		-> Don't attempt to build this module
#	package		-> Used in "sample" implementation of rpm packages
#			   from git snapshots

my $final_destdir = $rpm ? $package_destdir : $xorg_git_dest_dir;

my $ac_libdir     = "--libdir=$final_destdir/$xorg_git_lib_name";

#xedit
my $ac_noxprint	  = "--disable-xprint";

# libX11
my $ac_xcb	  = "--with-xcb=yes";

# xserver
my $ac_buildaddr  = "--with-builder-addr="			.
    ($rpm ? "\$USER\@\`hostname\`" : $git_user_email);
my $ac_builderstr;
my $ac_kvesa;
if ($rpm) {
    ## FIXME did not build on buildsystem due to missing IOPL_MASK definition...
    $ac_kvesa	  = " \\
%ifarch %%{x86} alpha
--disable-kdrive-vesa \\
%endif
";
    $ac_builderstr = "--with-builderstring=\"\`uname -onri | sed -e 's/ /_/g'\`\"";
}
else {
    $ac_kvesa	   = $xorg_git_lib_name eq "lib" ? "" : "--disable-kdrive-vesa";
    $ac_builderstr = `uname -a`; chomp($ac_builderstr); $ac_builderstr = "--with-builderstring=\"\\\"$ac_builderstr\\\"\"";
}

my $ac_dga	  = "--enable-dga";
my $ac_dmx	  = "--disable-dmx";
my $ac_dri2	  = "--enable-dri2";
my $ac_fontcache  = "--enable-fontcache";
my $ac_fonts	  = "--disable-builtin-fonts";
my $ac_glx	  = "--enable-glx";
my $ac_kdrive	  = "--disable-kdrive";
my $ac_log_dir	  = "--with-log-dir=$final_destdir/var/log";
my $ac_xace	  = "--enable-xace";
my $ac_xdmcp	  = "--enable-xdmcp";
my $ac_xdmauth	  = "--enable-xdm-auth-1";
my $ac_xephyr	  = "--disable-xephyr";
my $ac_xf86misc	  = "--enable-xf86misc";
my $ac_xf86vid	  = "--enable-xf86vidmode";
my $ac_xfake	  = "--disable-xfake";
my $ac_xfbdev	  = "--disable-xfbdev";
my $ac_xorg	  = "--enable-xorg";
my $ac_xorgcfg	  = "--enable-xorgcfg";
my $ac_xsdl	  = "--disable-xsdl";
my $ac_xselinux	  = "--disable-xselinux";
my $ac_xvfb	  = "--enable-xvfb";

# drm kernel modules to build
my @drm_modules = ("drm", "i810", "i915", "mach64", "mga",
		   "r128", "radeon", "sis", "tdfx", "savage", "via");

my $ac_glxtls	  = "--disable-glx-tls";

# mesa
my $ac_demos	  = "--with-demos=yes";
my $ac_mesadriver = "--with-driver=dri";
my $ac_mesadridir = "--with-dri-driverdir=$final_destdir/$xorg_git_lib_name/dri";
my $ac_ttm	  = "--enable-ttm-api";
my $ac_x	  = "--with-x";

# mesa
my $ac_xincludes  = "--x-includes=$final_destdir/include/X11";
my $ac_xlibraries = "--x-libraries=$final_destdir/$xorg_git_lib_name";

# xserver, mesa
my $ac_debug	  = "--enable-debug";

# xphelloworld
my $ac_motiflibs  = "--with-motif-libraries=no";

# compiz
my $ac_kde4	  = "--disable-kde --disable-kde4 --disable-kconfig";
my $ac_kconfig	  = $rpm ? "--disable-kconfig" : "";
my $ac_gnome	  = $rpm ? "--disable-gnome" : "";

# xkeyboard-config
my $ac_xkbcomplnk = "--disable-xkbcomp-symlink";
my $ac_xkbrules   = "--enable-compat-rules";
my $ac_xkbruleslnk= "--with-xkb-rules-symlink=xorg";

# For tarballs
my $tgz		= ".tar.bz2";
my $xorg_app	= "app";
my $xorg_doc	= "doc";
my $xorg_driver	= "driver";
my $xorg_font	= "font";
my $xorg_lib	= "lib";
my $xorg_proto	= "proto";
my $xorg_url	= "http://xorg.freedesktop.org/releases/individual";
my $xorg_util	= "util";


my %branches =
(
    'branch-gem'	=>
    {
	'packages'	=> ["libdrm", "mesa", "xf86-video-intel"],
    },
    'X11R7.4'		=>
    {
	'packages'	=> ["libdrm", "mesa", "xserver"],
    },
    'branch-glucose'	=>
    {
	'packages'	=> ["xserver"],
    },
);

my @modules =
(
 {
     'name'	=> "modular",
     'path'	=> "xorg/util",
     'version'	=> "0.0.1",
     # Fixme tarball
     'skip'	=> "yes",
 },
 {
     'name'	=> "macros",
     'path'	=> "xorg/util",
     'version'	=> "1.1.6",
     'tarball'	=> "$xorg_url/$xorg_util/util-%{name}-%{version}$tgz",
     'package'	=> "xorg-macros",
 },
 {
     'name'	=> "x11proto",
     'path'	=>"xorg/proto",
     'version'	=> "7.0.12",
     'tarball'	=> "$xorg_url/$xorg_proto/xproto-%{version}$tgz",
     'package'	=> "x11proto",
 },
 {
     'name'	=> "libxtrans",
     'path'	=> "xorg/lib",
     'version'	=> "1.2",
     'tarball'	=> "$xorg_url/$xorg_lib/xtrans-%{version}$tgz",
     'package'	=> "libxtrans",
 },
 {
     'name'	=> "libXau",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxau",
 },
 {
     'name'	=> "libXdmcp",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxdmcp",
 },
 {
     'name'	=> "pthread-stubs",
     'path'	=> "xcb",
     'version'	=> "0.1",
     'tarball'	=> "$xorg_url/$xorg_lib/libpthread-stubs-%{version}$tgz",
     'package'	=> "pthread-stubs",
 },
 {
     'name'	=> "proto",
     'path'	=> "xcb",
     'version'	=> "1.1",
     'tarball'	=> "http://xcb.freedesktop.org/dist/%{path}-%{name}-%{version}$tgz",
     'package'	=> "xcb-proto",
     'build'	=> ["libxml2-utils", "python"],
 },
 {
     'name'	=> "libICE",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libice",
 },
 {
     'name'	=> "inputproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.4.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "inputproto",
 },
 {
     'name'	=> "kbproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "kbproto",
 },
 {
     'name'	=> "xf86bigfontproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xf86bigfontproto",
 },
 {
     'name'	=> "libxcb",
     'path'	=> "xcb",
     'version'	=> "1.1",
     'tarball'	=> "http://xcb.freedesktop.org/dist/%{name}-%{version}$tgz",
     'package'	=> "libxcb",
     'build'	=> ["libxslt-proc", "python", "doxygen", "graphviz",
		    "pthread-stubs$package_suffix",
		    "libx11$package_suffix", "libxext$package_suffix"],
     'requires' => ["pthread-stubs$package_suffix"],
 },
 {
     'name'	=> "xextproto",
     'path'	=>"xorg/proto",
     'version'	=> "7.0.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xextproto",
 },
 {
     'name'	=> "libSM",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libsm",
		    # uuid
     'build'	=> ["libext2fs-devel"],
 },
 {
     'name'	=> "bigreqsproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "bigreqsproto",
 },
 {
     'name'	=> "xcmiscproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xcmiscproto",
 },
 {
     'name'	=> "libX11",
     'path'	=> "xorg/lib",
     'version'	=> "1.1.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'configure'=> "$ac_xcb",
     'package'	=> "libx11",
     'build'	=> ["xcmiscproto$package_suffix", "bigreqsproto$package_suffix",
		    "libxcb$package_suffix"],
 },
 {
     'name'	=> "libXt",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.5",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxt",
 },
 {
     'name'	=> "libXext",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxext",
 },
 {
     'name'	=> "printproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "printproto",
 },
 {
     'name'	=> "libXmu",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxmu",
 },
 {
     'name'	=> "libXpm",
     'path'	=> "xorg/lib",
     'version'	=> "3.5.7",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxpm",
     'requires'	=> ["libxcb$package_suffix"],
 },
 {
     'name'	=> "libXp",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxp",
 },
 {
     'name'	=> "libXaw",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxaw",
 },
 {
     'name'	=> "util",
     'path'	=> "xorg/font",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-util",
 },
 {
     'name'	=> "fixesproto",
     'path'	=>"xorg/proto",
     'version'	=> "4.0",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "fixesproto",
 },
 {
     'name'	=> "renderproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.9.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "renderproto",
 },
 {
     'name'	=> "libXfixes",
     'path'	=> "xorg/lib",
     'version'	=> "4.0.3",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxfixes",
 },
 {
     'name'	=> "compositeproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.4",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "compositeproto",
 },
 {
     'name'	=> "libXrender",
     'path'	=> "xorg/lib",
     'version'	=> "0.9.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxrender",
 },
 {
     'name'	=> "damageproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "damageproto",
 },
 {
     'name'	=> "xineramaproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xineramaproto",
 },
 {
     'name'	=> "randrproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.2.1",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "randrproto",
 },
 {
     'name'	=> "libxkbfile",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.5",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxkbfile",
 },
 {
     'name'	=> "videoproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.2.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "videoproto",
 },
 {
     'name'	=> "fontcacheproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.1.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "fontcacheproto",
 },
 {
     'name'	=> "fontsproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.0.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "fontsproto",
 },
 {
     'name'	=> "libfontenc",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libfontenc",
     'build'	=> ["zlib-devel"],
 },
 {
     'name'	=> "libXcomposite",
     'path'	=> "xorg/lib",
     'version'	=> "0.4.0",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxcomposite",
     'build'	=> ["xmlto"],
 },
 {
     'name'	=> "libXcursor",
     'path'	=> "xorg/lib",
     'version'	=> "1.1.9",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxcursor",
 },
 {
     'name'	=> "libXdamage",
     'path'	=> "xorg/lib",
     'version'	=> "1.1.1",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxdamage",
 },
 {
     'name'	=> "libXi",
     'path'	=> "xorg/lib",
     'version'	=> "1.1.3",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxi",
     'build'	=> ["xmlto", "docbook-dtd43-xml"],
 },
 {
     'name'	=> "libXinerama",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxinerama",
 },
 {
     'name'	=> "libXrandr",
     'path'	=> "xorg/lib",
     'version'	=> "1.2.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxrandr",
 },
 {
     'name'	=> "xf86miscproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.9.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xf86miscproto",
 },
 {
     'name'	=> "xf86vidmodeproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.2.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xf86vidmodeproto",
 },
 {
     'name'	=> "dmxproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.2.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "dmxproto",
 },
 {
     'name'	=> "resourceproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "resourceproto",
 },
 {
     'name'	=> "recordproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.13.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "recordproto",
 },
 {
     'name'	=> "glproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.4.9",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "glproto",
 },
 {
     'name'	=> "xf86driproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.0.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xf86driproto",
 },
 {
     'name'	=> "libxkbui",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxkbui",
 },
 {
     'name'	=> "evieproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_proto/evieext-%{version}$tgz",
     'package'	=> "evieproto",
 },
 {
     'name'	=> "libXv",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxv",
 },
 {
     'name'	=> "dri2proto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "dri2proto",
 },
 {
     'name'	=> "scrnsaverproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "scrnsaverproto",
 },
 {
     'name'	=> "xf86dgaproto",
     'path'	=>"xorg/proto",
     'version'	=> "2.0.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xf86dgaproto",
 },
 {
     'name'	=> "trapproto",
     'path'	=>"xorg/proto",
     'version'	=> "3.4.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "trapproto",
 },
 {
     'name'	=> "libXfont",
     'path'	=> "xorg/lib",
     'version'	=> "1.3.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxfont",
     'build'	=> ["freetype2-devel"],
     'requires'	=> ["freetype2-devel"],
 },
 {
     'name'	=> "libpciaccess",
     'path'	=> "xorg/lib",
     'version'	=> "0.10",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libpciaccess",
     'build'	=> ["pciids"],
 },
 {
     'name'	=> "pixman",
     'path'	=> "",	# empty path
     'version'	=> "0.9.6",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "pixman",
 },
 {
     'name'	=> "libXxf86misc",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxxf86misc",
 },
 {
     'name'	=> "libXxf86vm",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxxf86vm",
 },
 {
     ## XXX Possible cyclic dependency.
     ## Build of xserver was using system installed version, but
     ## xserver with dmx enabled requires pkgconfig dmx, while
     ## libdmx requires xserver built...
     'name'	=> "libdmx",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libdmx",
 },

 {
     'name'	=> "drm",
     'path'	=> "mesa",
     'version'	=> "2.3.0",
     'tarball'	=> "http://dri.freedesktop.org/libdrm/libdrm-%{version}$tgz",
     'package'	=> "libdrm",
 },
 {
     'name'	=> "drm",
     'path'	=> "mesa",
     'commit'	=> "origin/drm-gem",
     'package'	=> "libdrm-branch-gem",
 },
 {
     'name'	=> "drm",
     'path'	=> "mesa",
     'commit'	=> "origin/libdrm-2_3-branch",
     'package'	=> "libdrm-X11R7.4",
 },

 {
     'name'	=> "makedepend",
     'path'	=> "xorg/util",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_util/%{name}-%{version}$tgz",
     'package'	=> "makedepend",
 },
 {
     'name'	=> "gccmakedep",
     'path'	=> "xorg/util",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_util/%{name}-%{version}$tgz",
     'package'	=> "gccmakedep",
 },

 {
     'name'	=> "mesa",
     'path'	=> "mesa",
     'version'	=> "7.0.3",
     # Fixme tarball
     'configure'=>
     "$ac_demos "	.
     "$ac_glxtls "	.
     "$ac_mesadriver "	.
     "$ac_mesadridir "	.
     "MKDEP=$final_destdir/bin/gccmakedep "	.
     "$ac_ttm "		.
     "$ac_x "		.
     "$ac_xincludes "	.
     "$ac_xlibraries "	.
     ($options {'d'} ? "$ac_debug" : ""),
     'package'	=> "mesa",
     'build'	=> ["tcl", "texinfo", "libexpat-devel",
		    "gccmakedep$package_suffix",
		    "pixman$package_suffix",
		    "libpciaccess$package_suffix",
		    ],
 },
 {
     'name'	=> "mesa",
     'path'	=> "mesa",
     # Fixme tarball
     'configure'=>
     "$ac_demos "	.
     "$ac_glxtls "	.
     "--with-dri-drivers=swrast,i915,i965 " .
     "$ac_mesadriver "	.
     "$ac_mesadridir "	.
     "$ac_ttm "		.
     "$ac_x "		.
     "$ac_xincludes "	.
     "$ac_xlibraries "	.
     ($options {'d'} ? "$ac_debug" : ""),
     'commit'	=> "origin/drm-gem",
     'package'	=> "mesa-branch-gem",
     'build'	=> ["tcl", "texinfo", "libexpat-devel",
		    "gccmakedep$package_suffix",
		    "pixman$package_suffix",
		    "libpciaccess$package_suffix",
		    ],
 },
 {
     'name'	=> "mesa",
     'path'	=> "mesa",
     # Fixme tarball
     'package'	=> "mesa-X11R7.4",
     'commit'	=> "origin/gallium-0.1",
     'build'	=> ["tcl", "texinfo", "libexpat-devel",
		    "gccmakedep$package_suffix",
		    "pixman$package_suffix",
		    "libpciaccess$package_suffix",
		    ],
 },

 {
     ## XXX Another possible cyclic dependency.
     ## Build of xserver was using system installed version, but
     ## configure requires xres package available, and libxres
     ## requires xserver built...
     'name'	=> "libXRes",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_lib/libXres-%{version}$tgz", # lowercase r
     'package'	=> "libxres",
 },
 {
     'name'	=> "libXtst",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxtst",
     'build'	=> ["xmlto"],
 },

 {
     'name'	=> "xserver",
     'path'	=> "xorg",
     'version'	=> "1.5.99.901",
     'tarball'	=> "$xorg_url/xserver/xorg-server-%{version}$tgz",
     'configure'=>
     "$ac_buildaddr "	.
     "$ac_builderstr "	.
     "$ac_dga "		.
     "$ac_dmx "		.
     "$ac_dri2 "	.
     "$ac_fonts "	.
     "$ac_glx "		.
     "$ac_kdrive "	.
     "$ac_kvesa "	.
     "$ac_log_dir "	.
     "$ac_xace "	.
     "$ac_xdmcp "	.
     "$ac_xdmauth "	.
     "$ac_xephyr "	.
     "$ac_xf86vid "	.
     "$ac_xfake "	.
     "$ac_xfbdev "	.
     "$ac_xorg "	.
     "$ac_xsdl "	.
     "$ac_xselinux "	.
     "$ac_xvfb "	.
     ($options{'d'} ? "$ac_debug" : ""),
     'package'	=> "xserver",
     'build'	=> ["libpam-devel", "libgpm-devel",
		    "libhal-devel", "libdbus-devel",
		    "openssl-devel", "bison", "flex",
		    # verify -- required by configure
		    "libxres$package_suffix",
		    "dmxproto$package_suffix", "libdmx$package_suffix"],
     'requires'	=> ["libpciaccess$package_suffix", "freetype2-devel"],
 },

 {
     'name'	=> "glitz",
     'path'	=> "",
     'version'	=> "0.5.6",
     'tarball'	=> "http://cairographics.org/snapshots/%{name}-{%version}$tgz",
     'package'	=> "libglitz",
     'skip'	=> "used only by glucose",
 },

 {
     'name'	=> "xserver",
     'path'	=> "xorg",
     'configure'=>
     "$ac_buildaddr "	.
     "$ac_builderstr "	.
     "$ac_dga "		.
     "$ac_dmx "		.
     "$ac_dri2 "	.
     "$ac_glx "		.
     "$ac_glxtls "	.
     "$ac_kdrive "	.
     "$ac_kvesa "	.
     "$ac_log_dir "	.
     "$ac_xace "	.
     "$ac_xdmcp "	.
     "$ac_xdmauth "	.
     "$ac_xephyr "	.
     "$ac_xf86misc "	.
     "$ac_xf86vid "	.
     "$ac_xfake "	.
     "$ac_xfbdev "	.
     "$ac_xorg "	.
     "$ac_xorgcfg "	.
     "$ac_xsdl "	.
     "$ac_xselinux "	.
     "$ac_xvfb "	.
     ($options{'d'} ? "$ac_debug" : ""),
     'commit'	=> "origin/server-1.5-branch",
     'package'	=> "xserver-X11R7.4",
     'build'	=> ["libpam-devel", "libgpm-devel",
		    "libhal-devel", "libdbus-devel",
		    "openssl-devel", "bison", "flex",
		    # verify -- required by configure
		    "libxres$package_suffix",
		    "dmxproto$package_suffix", "libdmx$package_suffix" ],
     'requires'	=> ["libpciaccess$package_suffix", "freetype2-devel"],
 },

 {
     'name'	=> "xserver",
     'path'	=> "xorg",
     'configure'=>
     "$ac_buildaddr "	.
     "$ac_builderstr "	.
     "$ac_fontcache "	.
     "$ac_glx "		.
     "$ac_glxtls "	.
     "$ac_log_dir "	.
     "$ac_xdmcp "	.
     "$ac_xdmauth "	.
     ($options{'d'} ? "$ac_debug" : ""),
     'commit'	=> "origin/glucose-2",
     'package'	=> "xserver-branch-glucose",
     'build'	=> ["libpam-devel", "libgpm-devel",
		    "libgii-devel", "libhal-devel", "libdbus-devel",
		    "openssl-devel", "bison", "flex",
		    # verify -- required by configure
		    "libxres$package_suffix",
		    "dmxproto$package_suffix", "libdmx$package_suffix",
		    ],
     'requires'	=> ["libpciaccess$package_suffix","freetype2-devel"],
     'skip'	=> "not building",
 },

 {
     'name'	=> "xprint",
     'path'	=> "xorg",
     'version'	=> "1.5.99.1",
     'skip'	=> "yes",
 },
 {
     'name'	=> "libXprintUtil",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxprintutil",
 },
 {
     'name'	=> "fontconfig",
     'path'	=> "",	# empty path
     'version'	=> "2.5.91",
     'tarball'	=> "http://fontconfig.org/release/fontconfig-%{version}.tar.gz",
     'package'	=> "fontconfig",
     'build'	=> ["ed", "lynx",
		    "freetype2-devel", "docbook-utils", "docbook-utils-pdf",
		    "docbook-dtd31-sgml", "docbook-dtd41-sgml",
		    "libxml2-devel", "libxml2-utils"],
     'skip'	=> "Not used in Mandriva repositories",
 },
 {
     'name'	=> "windowswmproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "windowswmproto",
 },
 {
     'name'	=> "applewmproto",
     'path'	=> "xorg/proto",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "applewmproto",
 },
 {
     'name'	=> "mkfontdir",
     'path'	=> "xorg/app",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "mkfontdir",
 },
 {
     'name'	=> "mkfontscale",
     'path'	=> "xorg/app",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "mkfontscale",
     'build'	=> ["freetype2-devel"],
 },
 {
     'name'	=> "bdftopcf",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "bdftopcf",
     'build'	=> ["freetype2-devel"],
 },
 {
     'name'	=> "libXvMC",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxvmc",
 },
 {
     'name'	=> "xcursorgen",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xcursorgen",
     'build'	=> ["libpng-devel"],
 },
 {
     'name'	=> "libXTrap",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxtrap",
 },
 {
     'name'	=> "bitmaps",
     'path'	=> "xorg/data",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/data/xbitmaps-1.0.1$tgz",
     'package'	=> "xorg-data-bitmaps",
 },
 {
     'name'	=> "libXfontcache",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxfontcache",
 },
 {
     'name'	=> "pmproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_proto/xproxymanagementprotocol-%{version}$tgz",
     'package'	=> "pmproto",
 },
 {
     'name'	=> "libXprintAppUtil",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxprintapputil",
     'requires'	=> ["libxprintutil$package_suffix"],
 },
 {
     'name'	=> "libXft",
     'path'	=> "xorg/lib",
     'version'	=> "2.1.12",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxft",
     'build'	=> ["fontconfig-devel"],
 },
 {
     'name'	=> "libFS",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libfs",
 },
 {
     'name'	=> "libXxf86dga",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxxf86dga",
 },
 {
     'name'	=> "liblbxutil",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "liblbxutil",
     'build'	=> ["zlib-devel"],
 },
 {
     'name'	=> "libXScrnSaver",
     'path'	=> "xorg/lib",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxscrnsaver",
 },
 {
     'name'	=> "util",
     'path'	=> "xcb",
     'version'	=> "0.2",
     'tarball'	=> "http://xcb.freedesktop.org/dist/%{path}-%{name}-%{version}$tgz",
     'package'	=> "xcb-util",
     'build'	=> ["gperf"],
 },
 {
     'name'	=> "lndir",
     'path'	=> "xorg/util",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_util/%{name}-%{version}$tgz",
     'package'	=> "lndir",
 },
 {
     'name'	=> "imake",
     'path'	=> "xorg/util",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_util/%{name}-%{version}$tgz",
     'package'	=> "imake",
 },
 {
     'name'	=> "cf",
     'path'	=> "xorg/util",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_util/xorg-cf-files-%{version}$tgz",
     'package'	=> "xorg-util-cf",
 },
 {
     'name'	=> "xf86rushproto",
     'path'	=>"xorg/proto",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_proto/%{name}-%{version}$tgz",
     'package'	=> "xf86rushproto",
 },
 {
     'name'	=> "liboldX",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "liboldx",
 },
 {
     'name'	=> "libXevie",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libxevie",
 },
 {
     'name'	=> "libWindowsWM",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libwindowswm",
 },
 {
     'name'	=> "libAppleWM",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_lib/%{name}-%{version}$tgz",
     'package'	=> "libapplewm",
 },
 {
     'name'	=> "xfree86-type1",
     'path'	=> "xorg/font",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-xfree86-type1",
     'build'	=> ["fontconfig-devel",
		    "mkfontdir$package_suffix", "mkfontscale$package_suffix"],
 },
 {
     'name'	=> "winitzki-cyrillic",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-winitzki-cyrillic",
 },
 {
     'name'	=> "sony-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-sony-misc",
 },
 {
     'name'	=> "screen-cyrillic",
     'path'	=> "xorg/font",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-screen-cyrillic",
 },
 {
     'name'	=> "schumacher-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-schumacher-misc",
 },
 {
     'name'	=> "misc-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-misc-misc",
 },
 {
     'name'	=> "misc-meltho",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-misc-meltho",
     'build'	=> ["fontconfig",
		    "mkfontdir$package_suffix", "mkfontscale$package_suffix"],
 },
 {
     'name'	=> "misc-ethiopic",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-misc-ethiopic",
     'build'	=> ["fontconfig",
		    "mkfontdir$package_suffix", "mkfontscale$package_suffix"],
 },
 {
     'name'	=> "misc-cyrillic",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-misc-cyrillic",
 },
 {
     'name'	=> "micro-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-micro-misc",
 },
 {
     'name'	=> "jis-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-jis-misc",
 },
 {
     'name'	=> "isas-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-isas-misc",
 },
 {
     'name'	=> "ibm-type1",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-ibm-type1",
     'build'	=> ["fontconfig",
		    "mkfontdir$package_suffix", "mkfontscale$package_suffix"],
 },
 {
     'name'	=> "encodings",
     'path'	=> "xorg/font",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_font/%{name}-%{version}$tgz",
     'package'	=> "font-encodings",
 },
 {
     'name'	=> "dec-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-dec-misc",
 },
 {
     'name'	=> "daewoo-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-daewoo-misc",
 },
 {
     'name'	=> "cursor-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-cursor-misc",
 },
 {
     'name'	=> "cronyx-cyrillic",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-cronyx-cyrillic",
 },
 {
     'name'	=> "bitstream-type1",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bitstream-type1",
     'build'	=> ["fontconfig",
		    "mkfontdir$package_suffix", "mkfontscale$package_suffix"],
 },
 {
     'name'	=> "bitstream-speedo",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bitstream-speedo",
 },
 {
     'name'	=> "bitstream-75dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bitstream-75dpi",
 },
 {
     'name'	=> "bitstream-100dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bitstream-100dpi",
 },
 {
     'name'	=> "bh-type1",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bh-type1",
     'build'	=> ["fontconfig",
		    "mkfontdir$package_suffix", "mkfontscale$package_suffix"],
 },
 {
     'name'	=> "bh-ttf",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bh-ttf",
     'build'	=> ["fontconfig",
		    "mkfontdir$package_suffix", "mkfontscale$package_suffix"],
 },
 {
     'name'	=> "bh-lucidatypewriter-75dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bh-lucidatypewriter-75dpi",
 },
 {
     'name'	=> "bh-lucidatypewriter-100dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bh-lucidatypewriter-100dpi",
 },
 {
     'name'	=> "bh-75dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bh-75dpi",
 },
 {
     'name'	=> "bh-100dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-bh-100dpi",
 },
 {
     'name'	=> "arabic-misc",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-arabic-misc",
 },
 {
     'name'	=> "alias",
     'path'	=> "xorg/font",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-alias",
 },
 {
     'name'	=> "adobe-utopia-75dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-adobe-utopia-75dpi",
 },
 {
     'name'	=> "adobe-utopia-100dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-adobe-utopia-100dpi",
 },
 {
     'name'	=> "adobe-75dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-adobe-75dpi",
 },
 {
     'name'	=> "adobe-100dpi",
     'path'	=> "xorg/font",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_font/font-%{name}-%{version}$tgz",
     'package'	=> "font-adobe-100dpi",
 },
 {
     ### FIXME need to find official tarball url, if any
     'name'	=> "xf86-video-xgixp",
     'path'	=> "xorg/driver",
     'version'	=> "1.7.99.3",
     'package'	=> "xf86-video-xgixp",
 },
 {
     'name'	=> "xf86-video-xgi",
     'path'	=> "xorg/driver",
     'version'	=> "1.5.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-xgi",
 },
 {
     'name'	=> "xf86-video-wsfb",
     'path'	=> "xorg/driver",
     'version'	=> "0.2.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "*BSD only - wsfb_driver.c:46:33: error: dev/wscons/wsconsio.h: No such file or directory",
 },
 {
     'name'	=> "xf86-video-voodoo",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-voodoo",
 },
 {
     'name'	=> "xf86-video-vmware",
     'path'	=> "xorg/driver",
     'version'	=> "10.16.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-vmware",
 },
 {
     'name'	=> "xf86-video-via",
     'path'	=> "xorg/driver",
     'version'	=> "0.2.2",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "not building - drm/drm.h int64_t not defined - needs pciaccess conversion after not including drm/drm.h",
     'package'	=> "xf86-video-via",
 },
 {
     'name'	=> "xf86-video-vga",
     'path'	=> "xorg/driver",
     'version'	=> "4.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-vga",
 },
 {
     'name'	=> "xf86-video-vermilion",
     'path'	=> "xorg/driver",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "vermilion.h:92: error: expected specifier-qualifier-list before 'pciVideoPtr'",
     'package'	=> "xf86-video-vermilion",
 },
 {
     'name'	=> "xf86-video-vesa",
     'path'	=> "xorg/driver",
     'version'	=> "1.3.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-vesa",
 },
 {
     'name'	=> "xf86-video-v4l",
     'path'	=> "xorg/driver",
     'version'	=> "0.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-v4l",
 },
 {
     'name'	=> "xf86-video-tseng",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-tseng",
 },
 {
     'name'	=> "xf86-video-trident",
     'path'	=> "xorg/driver",
     'version'	=> "1.3.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-trident",
 },
 {
     'name'	=> "xf86-video-tga",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-tga",
 },
 {
     'name'	=> "xf86-video-tdfx",
     'path'	=> "xorg/driver",
     'version'	=> "1.4.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-tdfx",
 },
 {
     'name'	=> "xf86-video-suntcx",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "sparc architecture",
 },
 {
     'name'	=> "xf86-video-sunleo",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "sparc architecture",
 },
 {
     'name'	=> "xf86-video-sunffb",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "sparc architecture",
 },
 {
     'name'	=> "xf86-video-suncg6",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "sparc architecture",
 },
 {
     'name'	=> "xf86-video-suncg3",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "sparc architecture",
 },
 {
     'name'	=> "xf86-video-suncg14",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "sparc architecture",
 },
 {
     'name'	=> "xf86-video-sunbw2",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "sparc architecture",
 },
 {
     'name'	=> "xf86-video-sisusb",
     'path'	=> "xorg/driver",
     'version'	=> "0.9.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-sisusb",
 },
 {
     'name'	=> "xf86-video-sis",
     'path'	=> "xorg/driver",
     'version'	=> "0.10.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-sis",
     'build'	=> ["inputproto$package_suffix"],
 },
 {
     'name'	=> "xf86-video-siliconmotion",
     'path'	=> "xorg/driver",
     'version'	=> "1.6.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-siliconmotion",
 },
 {
     'name'	=> "xf86-video-savage",
     'path'	=> "xorg/driver",
     'version'	=> "2.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-savage",
 },
 {
     'name'	=> "xf86-video-s3virge",
     'path'	=> "xorg/driver",
     'version'	=> "1.10.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-s3virge",
 },
 {
     'name'	=> "xf86-video-s3",
     'path'	=> "xorg/driver",
     'version'	=> "0.6.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-s3",
 },
 {
     'name'	=> "xf86-video-rendition",
     'path'	=> "xorg/driver",
     'version'	=> "4.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-rendition",
 },
 {
     'name'	=> "xf86-video-radeonhd",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-radeonhd",
     'build'	=> ["pciutils-devel"]
 },
 {
     'name'	=> "xf86-video-r128",
     'path'	=> "xorg/driver",
     'version'	=> "6.8.0",
     # not available
     #'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-r128",
 },
 {
     'name'	=> "xf86-video-nv",
     'path'	=> "xorg/driver",
     'version'	=> "2.1.9",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-nv",
 },
 {
     'name'	=> "xf86-video-nsc",
     'path'	=> "xorg/driver",
     'version'	=> "2.8.3",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-nsc",
 },
 {
     'name'	=> "xf86-video-newport",
     'path'	=> "xorg/driver",
     'version'	=> "0.2.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-newport",
 },
 {
     'name'	=> "xf86-video-neomagic",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-neomagic",
 },
 {
     'name'	=> "xf86-video-mga",
     'path'	=> "xorg/driver",
     'version'	=> "1.4.8",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-mga",
 },
 {
     'name'	=> "xf86-video-mach64",
     'path'	=> "xorg/driver",
     'version'	=> "6.8.0",
     # not yet available
     #'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-mach64",
 },
 {
     'name'	=> "xf86-video-intel",
     'path'	=> "xorg/driver",
     'version'	=> "2.3.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-intel",
 },

 {
     'name'	=> "xf86-video-intel",
     'path'	=> "xorg/driver",
     'commit'	=> "origin/drm-gem",
     'package'	=> "xf86-video-intel-branch-gem",
 },

 {
     'name'	=> "xf86-video-imstt",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "not building - imstt.h:13: error: expected specifier-qualifier-list before 'pciVideoPtr'",
     'package'	=> "xf86-video-imstt",
 },
 {
     'name'	=> "xf86-video-impact",
     'path'	=> "xorg/driver",
     'version'	=> "0.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "Not ix86: impact_shadow.c:17:26: error: asm/cachectl.h: No such file or directory",
 },
 {
     'name'	=> "xf86-video-i810",
     #'path'	=> "xorg/driver",
     'version'	=> "1.7.4",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "replaced by intel driver",
 },
 {
     'name'	=> "xf86-video-i740",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-i740",
 },
 {
     'name'	=> "xf86-video-i128",
     'path'	=> "xorg/driver",
     'version'	=> "1.3.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-i128",
 },
 {
     'name'	=> "xf86-video-glint",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-glint",
 },
 {
     'name'	=> "xf86-video-geode",
     'path'	=> "xorg/driver",
     'version'	=> "2.9.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-geode",
 },
 {
     'name'	=> "xf86-video-glide",
     'path'	=> "xorg/driver",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "Needs libglide & co",
 },
 {
     'name'	=> "xf86-video-fbdev",
     'path'	=> "xorg/driver",
     'version'	=> "0.4.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-fbdev",
 },
 {
     'name'	=> "xf86-video-dummy",
     'path'	=> "xorg/driver",
     'version'	=> "0.3.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-dummy",
 },
 {
     'name'	=> "xf86-video-cyrix",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "not building - cyrix.h:113: error: expected specifier-qualifier-list before 'pciVideoPtr'",
     'package'	=> "xf86-video-cyrix",
 },
 {
     'name'	=> "xf86-video-cirrus",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-cirrus",
 },
 {
     'name'	=> "xf86-video-chips",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-chips",
 },
 {
     'name'	=> "xf86-video-ati",
     'path'	=> "xorg/driver",
     'version'	=> "6.8.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-ati",
 },
 {
     'name'	=> "xf86-video-ast",
     'path'	=> "xorg/driver",
     'version'	=> "0.85.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-ast",
 },
 {
     'name'	=> "xf86-video-ark",
     'path'	=> "xorg/driver",
     'version'	=> "0.7.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-ark",
 },
 {
     'name'	=> "xf86-video-apm",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-video-apm",
 },
 {
     'name'	=> "xf86-video-amd",
     'path'	=> "xorg/driver",
     'version'	=> "2.7.7.7",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'skip'	=> "amd_gx_driver.c:331: error: 'pciVideoRec' undeclared (first use in this function)",
     'package'	=> "xf86-video-amd",
 },
 {
     'name'	=> "xf86-input-void",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-void",
 },
 {
     'name'	=> "xf86-input-vmmouse",
     'path'	=> "xorg/driver",
     'version'	=> "12.5.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-vmmouse",
 },
 {
     'name'	=> "xf86-input-ur98",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-ur98",
 },
 {
     'name'	=> "xf86-input-tek4957",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-tek4957",
 },
 {
     'name'	=> "xf86-input-synaptics",
     'path'	=> "xorg/driver",
     'version'	=> "0.15.0",
     #'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-synaptics",
     'build'	=> ["libxi$package_suffix", "kbproto$package_suffix"],
 },
 {
     'name'	=> "xf86-input-summa",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-summa",
     'skip'	=> "xf86Summa.c:1173: error: too many arguments to function 'InitValuatorClassDeviceStruct'",
 },
 {
     'name'	=> "xf86-input-spaceorb",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-spaceorb",
 },
 {
     'name'	=> "xf86-input-penmount",
     'path'	=> "xorg/driver",
     'version'	=> "1.3.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-penmount",
 },
 {
     'name'	=> "xf86-input-palmax",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-palmax",
 },
 {
     'name'	=> "xf86-input-mutouch",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-mutouch",
 },
 {
     'name'	=> "xf86-input-mouse",
     'path'	=> "xorg/driver",
     'version'	=> "1.3.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-mouse",
 },
 {
     'name'	=> "xf86-input-microtouch",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-microtouch",
 },
 {
     'name'	=> "xf86-input-magictouch",
     'path'	=> "xorg/driver",
     'version'	=> "1.0.0.5",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-magictouch",
 },
 {
     'name'	=> "xf86-input-magellan",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-magellan",
 },
 {
     'name'	=> "xf86-input-keyboard",
     'path'	=> "xorg/driver",
     'version'	=> "1.3.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-keyboard",
 },
 {
     'name'	=> "xf86-input-joystick",
     'path'	=> "xorg/driver",
     'version'	=> "1.3.2",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-joystick",
 },
 {
     'name'	=> "xf86-input-jamstudio",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-jamstudio",
 },
 {
     'name'	=> "xf86-input-hyperpen",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-hyperpen",
 },
 {
     'name'	=> "xf86-input-fpit",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-fpit",
 },
 {
     'name'	=> "xf86-input-evdev",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-evdev",
     'build'	=> ["libxi$package_suffix"],
 },
 {
     'name'	=> "xf86-input-elographics",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.0",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-elographics",
 },
 {
     'name'	=> "xf86-input-elo2300",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-elo2300",
 },
 {
     'name'	=> "xf86-input-dynapro",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-dynapro",
 },
 {
     'name'	=> "xf86-input-dmc",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-dmc",
 },
 {
     'name'	=> "xf86-input-digitaledge",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-digitaledge",
 },
 {
     'name'	=> "xf86-input-citron",
     'path'	=> "xorg/driver",
     'version'	=> "2.2.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-citron",
 },
 {
     'name'	=> "xf86-input-calcomp",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.2",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-calcomp",
 },
 {
     'name'	=> "xf86-input-aiptek",
     'path'	=> "xorg/driver",
     'version'	=> "1.1.1",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-aiptek",
 },
 {
     'name'	=> "xf86-input-acecad",
     'path'	=> "xorg/driver",
     'version'	=> "1.2.2",
     'tarball'	=> "$xorg_url/$xorg_driver/%{name}-%{version}$tgz",
     'package'	=> "xf86-input-acecad",
 },
 {
     'name'	=> "xorg-sgml-doctools",
     'path'	=> "xorg/doc",
     'version'	=> "1.2",
     'tarball'	=> "$xorg_url/$xorg_doc/%{name}-%{version}$tgz",
     'package'	=> "xorg-sgml-doctools",
 },
 {
     'name'	=> "xorg-docs",
     'path'	=> "xorg/doc",
     'version'	=> "1.4",
     'tarball'	=> "$xorg_url/$xorg_doc/%{name}-%{version}$tgz",
     'package'	=> "xorg-docs",
     'configure'=> "--enable-ps --enable-html --enable-txt",
     'build'	=> ["docbook-utils", "docbook-utils-pdf",
		    "linuxdoc-tools", "docbook-dtd31-sgml",
		    "docbook-dtd41-sgml", "docbook-dtd412-xml",
		    "docbook-dtd42-sgml", "docbook-dtd45-xml",
		    "docbook-dtd44-xml", "docbook-dtd43-xml",
		    "docbook-dtd42-xml",
		    "intltool", "poppler", "lynx", "texi2html",
		    "libxml2-devel", "libxml2-utils", "checkstyle", "xmltex"],
 },
 {
     'name'	=> "cursors",
     'path'	=> "xorg/data",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/data/xcursor-themes-1.0.1$tgz",
     'package'	=> "xorg-data-cursors",
 },
 {
     'name'	=> "xwud",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xwud",
 },
 {
     'name'	=> "xwininfo",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xwininfo",
 },
 {
     'name'	=> "xwd",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xwd",
 },
 {
     'name'	=> "xvinfo",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xvinfo",
 },
 {
     'name'	=> "xvidtune",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xvidtune",
 },
 {
     'name'	=> "xtrap",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xtrap",
 },
 {
     'name'	=> "xstdcmap",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xstdcmap",
 },
 {
     'name'	=> "xsm",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xsm",
	 # Shouldn't really need it to build from source...
     'build'	=> ["rsh"],
 },
 {
     'name'	=> "xsetroot",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xsetroot",
 },
 {
     'name'	=> "xsetpointer",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xsetpointer",
 },
 {
     'name'	=> "xsetmode",
     'path'	=> "xorg/app",
     'version'	=> "1.0.0",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xsetmode",
 },
 {
     'name'	=> "xset",
     'path'	=> "xorg/app",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xset",
 },
 {
     'name'	=> "xrx",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xrx",
 },
 {
     'name'	=> "xrestop",
     'path'	=> "xorg/app",
     'version'	=> "0.4",
     # Fixme tarball
     'package'	=> "xrestop",
     'build'	=> ["ncurses-devel"],
 },
 {
     'name'	=> "xrefresh",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xrefresh",
 },
 {
     'name'	=> "xrdb",
     'path'	=> "xorg/app",
     'version'	=> "1.0.5",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xrdb",
 },
 {
     'name'	=> "xrandr",
     'path'	=> "xorg/app",
     'version'	=> "1.2.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xrandr",
 },
 {
     'name'	=> "xprop",
     'path'	=> "xorg/app",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xprop",
 },
 {
     'name'	=> "xprehashprinterlist",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xprehashprinterlist",
 },
 {
     'name'	=> "xpr",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xpr",
 },
 {
     'name'	=> "xplsprinters",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xplsprinters",
     'requires'	=> ["libxprintutil$package_suffix",
		    "libxprintapputil$package_suffix"],
 },
 {
     'name'	=> "xphelloworld",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'configure'=> "$ac_motiflibs",
     'package'	=> "xphelloworld",
     'requires'	=> ["libxprintutil$package_suffix",
		    "libxprintapputil$package_suffix"],
 },
 {
     'name'	=> "xmore",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xmore",
     'requires'	=> ["libxprintutil$package_suffix",
		    "libxprintapputil$package_suffix"],
 },
 {
     'name'	=> "xmodmap",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xmodmap",
 },
 {
     'name'	=> "xmh",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xmh",
 },
 {
     'name'	=> "xmessage",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xmessage",
 },
 {
     'name'	=> "xman",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'configure'=> "--disable-xprint",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xman",
 },
 {
     'name'	=> "xmag",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xmag",
 },
 {
     'name'	=> "xlsfonts",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xlsfonts",
 },
 {
     'name'	=> "xlsclients",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xlsclients",
 },
 {
     'name'	=> "xlsatoms",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xlsatoms",
 },
 {
     'name'	=> "xlogo",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'configure'=> "--disable-xprint --with-render",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xlogo",
     'build'	=> ["fontconfig-devel"],
 },
 {
     'name'	=> "xload",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xload",
 },
 {
     'name'	=> "xkill",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xkill",
 },
 {
     'name'	=> "xkbutils",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xkbutils",
 },
 {
     'name'	=> "xkbprint",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xkbprint",
 },
 {
     'name'	=> "xkbevd",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xkbevd",
     'build'	=> ["bison", "flex"],
 },
 {
     'name'	=> "xkbcomp",
     'path'	=> "xorg/app",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xkbcomp",
     'build'	=> ["bison", "flex"],
 },
 {
     'name'	=> "xinput",
     'path'	=> "xorg/app",
     'version'	=> "1.3.0",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xinput",
 },
 {
     'name'	=> "xinit",
     'path'	=> "xorg/app",
     'version'	=> "1.0.8",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xinit",
 },
 {
     'name'	=> "xhost",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xhost",
 },
 {
     'name'	=> "xgc",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xgc",
     'build'	=> ["bison", "flex"],
 },
 {
     'name'	=> "xgamma",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xgamma",
 },
 {
     'name'	=> "xfwp",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xfwp",
 },
 {
     'name'	=> "xfsinfo",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xfsinfo",
 },
 {
     'name'	=> "xfs",
     'path'	=> "xorg/app",
     'version'	=> "1.0.6",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xfs",
 },
 {
     'name'	=> "xfontsel",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xfontsel",
 },
 {
     'name'	=> "xfindproxy",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xfindproxy",
 },
 {
     'name'	=> "xfd",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xfd",
     'build'	=> ["fontconfig-devel"],
 },
 {
     'name'	=> "xf86dga",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xf86dga",
 },
 {
     'name'	=> "xeyes",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xeyes",
 },
 {
     'name'	=> "xev",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xev",
 },
 {
     'name'	=> "xedit",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'configure'=> "$ac_noxprint",
     'package'	=> "xedit",
 },
 {
     'name'	=> "xdriinfo",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xdriinfo",
 },
 {
     'name'	=> "xdpyinfo",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xdpyinfo",
 },
 {
     'name'	=> "xdm",
     'path'	=> "xorg/app",
     'version'	=> "1.1.7",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xdm",
     'build'	=> ["fontconfig-devel"],
 },
 {
     'name'	=> "xditview",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xditview",
 },
 {
     'name'	=> "xdbedizzy",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xdbedizzy",
 },
 {
     'name'	=> "xconsole",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xconsole",
 },
 {
     'name'	=> "xcompmgr",
     'path'	=> "xorg/app",
     'version'	=> "1.1.4",
     'tarball'	=> "http://xapps.freedesktop.org/release/%{name}-%{version}.tar.gz",
     'package'	=> "xcompmgr",
 },
 {
     'name'	=> "xcmsdb",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xcmsdb",
 },
 {
     'name'	=> "xclock",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xclock",
     'build'	=> ["fontconfig-devel"],
 },
 {
     'name'	=> "xclipboard",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xclipboard",
 },
 {
     'name'	=> "xcalc",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xcalc",
 },
 {
     'name'	=> "xbiff",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xbiff",
 },
 {
     'name'	=> "xbacklight",
     'path'	=> "xorg/app",
     'version'	=> "1.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xbacklight",
 },
 {
     'name'	=> "xauth",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "xauth",
 },
 {
     'name'	=> "x11perf",
     'path'	=> "xorg/app",
     'version'	=> "1.5",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "x11perf",
     'build'	=> ["fontconfig-devel"],
 },
 {
     'name'	=> "viewres",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "viewres",
 },
 {
     'name'	=> "twm",
     'path'	=> "xorg/app",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "twm",
     'build'	=> ["bison", "flex"],
 },
 {
     'name'	=> "smproxy",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "smproxy",
 },
 {
     'name'	=> "showfont",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "showfont",
 },
 {
     'name'	=> "setxkbmap",
     'path'	=> "xorg/app",
     'version'	=> "1.0.4",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "setxkbmap",
 },
 {
     'name'	=> "sessreg",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "sessreg",
 },
 {
     'name'	=> "scripts",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     # Fixme tarball
     'package'	=> "xorg-scripts",
 },
 {
     'name'	=> "rstart",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "rstart",
 },
 {
     'name'	=> "rgb",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "rgb",
 },
 {
     'name'	=> "rendercheck",
     'path'	=> "xorg/app",
     'version'	=> "1.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "rendercheck",
 },
 {
     'name'	=> "proxymngr",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "proxymngr",
     'build'	=> ["liblbxutil$package_suffix", "lbxproxy$package_suffix" ],
 },
 {
     'name'	=> "oclock",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "oclock",
 },
 {
     'name'	=> "mkcomposecache",
     'path'	=> "xorg/app",
     'version'	=> "1.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "mkcomposecache",
 },
 {
     'name'	=> "luit",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "luit",
     'build'	=> ["zlib-devel"],
 },
 {
     'name'	=> "listres",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "listres",
 },
 {
     'name'	=> "lbxproxy",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "lbxproxy",
 },
 {
     'name'	=> "intel-gen4asm",
     'path'	=> "xorg/app",
     # Fixme tarball
     'package'	=> "intel-gen4asm",
     'build'	=> ["bison", "flex"],
 },
 {
     'name'	=> "ico",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "ico",
 },
 {
     'name'	=> "iceauth",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "iceauth",
 },
 {
     'name'	=> "grandr",
     'path'	=> "xorg/app",
     'version'	=> "0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "grandr",
     'build'	=> ["libgtk+2-devel", "libGConf2-devel"],
 },
 {
     'name'	=> "glxcompmgr",
     'path'	=> "xorg/app",
     # Fixme tarball
     'package'	=> "glxcompmgr",
     'build'	=> ["libGConf2-devel", "cairo-devel", "libpng-devel"],
 },
 {
     'name'	=> "fstobdf",
     'path'	=> "xorg/app",
     'version'	=> "1.0.2",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "fstobdf",
 },
 {
     'name'	=> "fslsfonts",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "fslsfonts",
 },
 {
     'name'	=> "fonttosfnt",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "fonttosfnt",
     'build'	=> ["freetype2-devel"],
 },
 {
     'name'	=> "fdclock",
     'path'	=> "xorg/app",
     # Fixme tarball
     'package'	=> "fdclock",
     'build'	=> ["libcairo-devel"],
 },
 {
     'name'	=> "editres",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "editres",
 },
 {
     'name'	=> "constype",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "constype",
 },
 {
     'name'	=> "compiz",
     'path'	=> "xorg/app",
     'version'	=> "0.7.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}.tar.gz",
     'package'	=> "compiz",
     'configure'=> "$ac_kde4 $ac_kconfig $ac_gnome",
     'build'	=> ["dbus-glib-devel", "libGConf2-devel",
		    "libgnome-window-settings-devel",
		    "libwnck-devel", "metacity-devel", "pango-devel",
		    "gnome-desktop-devel", "startup-notification-devel",
		    "kdebase4-workspace-devel",
		    "kde3-macros", "kdebase3-devel", "bonoboui-devel",
		    "libxslt-devel", "libxslt-proc", "librsvg-devel",
		    "libcairo-devel", "libsvg-cairo-devel",
		    "libdbus-qt-1-devel", "fuse-devel", "intltool"],
     'requires'	=> ["libxcb$package_suffix",],
 },
 {
     'name'	=> "bitmap",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "bitmap",
 },
 {
     'name'	=> "beforelight",
     'path'	=> "xorg/app",
     'version'	=> "1.0.3",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "beforelight",
 },
 {
     'name'	=> "appres",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'package'	=> "appres",
 },
 {
     'name'	=> "xkeyboard-config",
     'path'	=> "",
     'version'	=> "1.1",
     # Fixme tarball
     # 'commit'	=> "-r v_1_1 xkeyboard-config",
     'configure'=> "$ac_xkbcomplnk $ac_xkbrules $ac_xkbruleslnk",
     'package'	=> "xkeyboard-config",
     'build'	=> ["xkbcomp$package_suffix", "perl-XML-Parser",
		    "intltool", "glib-gettextize"],
 },
 {
     'name'	=> "demo",
     'path'	=> "xcb",
     'version'	=> "0.1",
     'tarball'	=> "http://xcb.freedesktop.org/dist/%{path}-%{name}-%{version}$tgz",
     'package'	=> "xcb-demo",
 },

 {
     'name'	=> "mesa",
     'path'	=> "nouveau",
     'commit'	=> "origin/gallium-0.1",
     'package'	=> "xf86-video-nouveau-mesa",
 },

 {
     'name'	=> "xf86-video-nouveau",
     'path'	=> "nouveau",
     'version'	=> "1.2.0",
     # Fixme tarball
     'package'	=> "xf86-video-nouveau",
 },

 {
     'name'	=> "xf86-video-avivo",
     'path'	=> "avivo",
     'version'	=> "0.0.999",
     # Fixme tarball
     'package'	=> "xf86-video-avivo",
     'requires'	=> "libpciaccess$package_suffix",
     ## actually broken compilation only
     'skip'	=> "../include/avivo.h:184: error: expected specifier-qualifier-list before 'pciVideoPtr'",
 },


 {
     'name'	=> "calibrateproto",
     'path'	=>"xorg/proto",
     'version'	=> "0.1.0",
     # no tarball at $xorg_url/$xorg_proto/
     'skip'	=> "ignore",
 },
 {
     'name'	=> "lg3dproto",
     'path'	=>"xorg/proto",
     'version'	=> "5.0",
     # no tarball available at $xorg_url/$xorg_proto/
     'skip'	=> "ignore",
 },
 {
     'name'	=> "libXamine",
     'path'	=> "xcb",
     'version'	=> "0.1",
     # Fixme tarball
     'skip'	=> "yes",
 },
 {
     'name'	=> "libXCalibrate",
     'path'	=> "xorg/lib",
     'version'	=> "0.4.0",
     # No tarball at $xorg_url/$xorg_lib/
     'skip'	=> "yes",
 },
 {
     'name'	=> "libXlg3d",
     'path'	=> "xorg/lib",
     'version'	=> "1.0.0",
     # No tarball at $xorg_url/$xorg_lib/
     'skip'	=> "yes",
 },
 # libxxf86rush unbuildable?
 #   tarball not available lib at $xorg_url/$xorg_lib/
 {
     'name'	=> "mkcfm",
     'path'	=> "xorg/app",
     'version'	=> "1.0.1",
     'tarball'	=> "$xorg_url/$xorg_app/%{name}-%{version}$tgz",
     'skip'	=> "broken missing symbols CIDOpenScalable, CIDRegisterFontFileFunctions (should be in libXfont ?)",
 },

 # x11-driver-input-sample unbuildable? or can be added?

 {
     'name'	=> "edid-decode",
     'path'	=> "xorg/app",
     # Fixme tarball
     # Needs an install target
     'skip'	=> "yes",
     'package'	=> "edid-decode",
     'build'	=> ["zsh"],
 },
 {
     'name'	=> "makepsres",
     'path'	=> "xorg/app",
     # Fixme tarball
     'skip'	=> "yes",
 },
 {
     'name'	=> "pclcomp",
     'path'	=> "xorg/app",
     # Fixme tarball
     'skip'	=> "yes",
 },
 {
     'name'	=> "xshowdamage",
     'path'	=> "xorg/app",
     # Fixme tarball
     # needs an install target
     'skip'	=> "yes",
 },
 {
     'name'	=> "install-check",
     'path'	=> "xorg/util",
     # No version
     # Fixme tarball
     'skip'	=> "yes",
 },
 {
     'name'	=> "xmkmf",
     'path'	=> "xorg/util",
     # No files
     # Fixme tarball
     'skip'	=> "yes",
 },
 );

##############################################################################
# Simple wrapper that prints errno string if not successful
sub x_system {
    my ($command, $die) = @_;

    print(STDERR "    $command\n");
    system($command);
    if ($?) {
	die(" ** Error code $? ($!)\n")
	    if ($die);
    }
};


##############################################################################
# Simple wrapper that prints errno string if not successful
sub x_chdir {
    my ($dir) = @_;

    print(STDERR "    cd $dir\n");
    chdir($dir) or die(" ** $!\n");
};


##############################################################################
# Does a clone/pull in the build dir to avoid poluting the main repository
# and giving a chance for better review of changes.
sub checkout {
    my ($package) = \%{$_};

    my ($path, $name, $commit) =
	($package->{'path'},
	 $package->{'name'},
	 $package->{'commit'});

    my $cvs = undef;
    my $command;
    my $prefix = ($path ne "" ? "$path/" : "") . $name;
    my $checkout = $commit;
    my $branch;

    if ($checkout =~ m/origin/) {
	$checkout =~ s/^origin\///;
	$branch = $checkout;
	$checkout = "$prefix-$checkout";
    }
    else {
	$checkout = $prefix;
    }

    x_chdir($xorg_git_build_dir);
    if (-d $checkout and not $commit) {
	x_chdir($checkout);

	# Special case
	if ($cvs) {
	    $command = "cvs up";
	}
	else {
	    $command = "git-pull";
	}

	# Follow pattern of other commands
	print(STDERR "    $command\n");
	my $result = `$command`;

	# Very basic way to know this directory needs to be recompiled
	if ($result !~ /Already up-to-date./) {
	    x_system("rm -f $build_ok");
	}
    }
    else {
	# Special case
	if ($cvs) {
	    x_system("cvs -d:pserver:anoncvs\@cvs.freedesktop.org:/cvs/$name login");
	    x_system("cvs -d:pserver:anoncvs\@cvs.freedesktop.org:/cvs/$name co $commit $name");
	}
	else {
	    if ($branch) {
		x_system("rm -fr $checkout");
		x_chdir("$xorg_git_checkout_dir/$path/$name");
		x_system("git branch -D $branch && git-gc");
		x_system("git checkout origin/$branch");
		x_system("git checkout -b $branch origin/$branch");
		x_system("git checkout master");
		x_chdir($xorg_git_build_dir);
	    }
	    x_system("mkdir -p `dirname $checkout`");
	    x_system("git-clone $xorg_git_checkout_dir/$prefix $checkout");
	    if ($commit ne "") {
		x_chdir($checkout);
		x_system("git-checkout $commit");
	    }
	}
    }
};


##############################################################################
# Updates git repository
sub update {
    my ($package) = \%{$_};

    my ($path, $name) =
	($package->{'path'},
	 $package->{'name'});

    x_system("echo -n $path/$name > $git_last_update");

    my $cvs = undef;
    my $command;
    my $prefix = ($path ne "" ? "$path/" : "") . $name;

    x_chdir($xorg_git_checkout_dir);
    if (-d $prefix) {
	x_chdir($prefix);

	# Special case
	if ($cvs) {
	    $command = "cvs up";
	}
	else {
	    if (`git diff`) {
		die("$name has uncommited changes.\n");
	    }
	    my $tmp = $/;
	    $/ = undef;
	    $command = `git branch`;
	    $/ = $tmp;
	    # Don't try to switch branches, as it may cause conflicting merges.
	    # Require that the package leave it in master branch, i.e. just
	    # in a state matching a standard/default readonly checkout.
	    if ($command !~ /^\* master$/m) {
		die("$path/$name is not in branch master.\n");
	    }
	    $command = "git-pull";
	}
    }
    else {
	# Special case
	if ($cvs) {
	    x_system("cvs -d:pserver:anoncvs\@cvs.freedesktop.org:/cvs/$name login");
	    $command = "cvs -d:pserver:anoncvs\@cvs.freedesktop.org:/cvs/$name co $package->{commit} $name";
	}
	else {
	    $command = "git clone $xorg_git_protocol://$xorg_git_repository/$prefix $prefix";
	}
    }
    x_system("$command");
};


##############################################################################
sub checktars {
    my ($package) = \%{$_};

    my ($path, $name, $tarball) =
	($package->{'path'},
	 $package->{'name'},
	 $package->{'tarball'});

    if ($tarball) {
    #	my $filename = File::Basename::basename($tarball);
    #
    #	if (not -f "$xorg_git_build_dir/$path/$name/$filename") {
    #	    x_chdir("$xorg_git_build_dir/$path/$name");
    #	    x_system("wget -q $tarball");
    #	    print(STDERR " ** Missing package downloaded from upstream.\n")
    #		unless ($?);
    #	}
    #	else {
	    x_system("wget --spider -q $tarball");
	#	}
	if ($?) {
	    print(STDERR " ** Error checking tarball $tarball.\n");
	}
    }
    else {
	print(STDERR " ** No tarball specified for package $name\n");
    }
};


##############################################################################
sub build {
    my ($package) = \%{$_};

    my ($path, $name, $configure, $commit) =
	($package->{'path'},
	 $package->{'name'},
	 $package->{'configure'},
	 $package->{'commit'});

    my $trace = $options{'g'} ? $xorg_trace_pl : "";
    $commit =~ s/^origin\///;
    $name = "$name-$commit" if ($commit ne "");
    my $full_path = "$xorg_git_build_dir/$path/$name";

    #if ($commit) {
	#x_system("rm -f $auto_cache_file");
    #}

    if (not -f "$full_path/$build_ok") {
	x_chdir("$full_path");

	my $arguments = " --prefix=$xorg_git_dest_dir --cache-file=$auto_cache_file $configure";

	# options{c} means continue for git clone and also used as a hint
	# to build to not restart from scratch.
	if (undef($options{'c'})) {
	    if (-f "Makefile") {
		x_system("make distclean");
	    }
	    if ($options{'g'}) {
		x_system("rm -f $name.{deps,files}{,.ext}");
	    }
	}

	if (-f "autogen.sh") {
	    x_system("$trace ./autogen.sh $arguments");
	}
	# Mesa special case only?
	elsif ($commit eq "") {
	    x_system("$trace make configure");
	    x_system("$trace ./configure $arguments");
	}

	if ($name =~ /^mesa-/) {
	    x_system("$trace make "				.
		     "OPT_FLAGS=\\\"-I$xorg_git_dest_dir/include -L$xorg_git_dest_dir/$xorg_git_lib_name $ENV{CFLAGS}\\\" ".
		     "X11_INCLUDES=-I$final_destdir/include "	.
		     "EXTRA_LIB_PATH=-L$final_destdir/lib "	.
		     "LIB_DIR=$xorg_git_lib_name "		.
		     "INCLUDE_DIR=$xorg_git_dest_dir/include "	.
		     "DRI_DRIVER_DIR=$xorg_git_dest_dir/$xorg_git_lib_name/dri " .
		     "MKDEP=$xorg_git_dest_dir/bin/gccmakedep "	.
		     "linux-dri");
	}
	else {
	    x_system("$trace "					.
		     ($options{'g'} ? "-l xorg-build.log " : "").
		     "make", 1);
	}

	if ($name =~ /^mesa-/) {
	    x_system("$xorg_git_sudo_command $trace "		.
		     ($options{'g'} ? "-i " : "")		.
		     "make "					.
		     "OPT_FLAGS=\\\"-I$xorg_git_dest_dir/include -L$xorg_git_dest_dir/$xorg_git_lib_name $ENV{CFLAGS}\\\" ".
		     "X11_INCLUDES=-I$final_destdir/include "	.
		     "EXTRA_LIB_PATH=-L$final_destdir/lib "	.
		     "INSTALL_DIR=$xorg_git_dest_dir "		.
		     "DRI_DRIVER_INSTALL_DIR=$xorg_git_dest_dir/$xorg_git_lib_name/dri " .
		     "install", 1);
	}
	else {
	    x_system("$xorg_git_sudo_command $trace "		.
		     ($options{'g'} ? "-i " : "")		.
		     "make install", 1);
	}

	if ($options{'g'}) {
	    x_system("$xorg_git_sudo_command mkdir -p $xorg_git_deps_dir/$path");
	    foreach my $ext ("deps", "files", "deps.ext", "files.ext") {
		x_system("$xorg_git_sudo_command "			.
			 "/bin/sh -c \"cat $name.$ext | sort | uniq > "	.
			 "$xorg_git_deps_dir/$path/$name.$ext\"");
	    }
	}

	x_system("touch $build_ok");
    }

    #if ($commit) {
	#x_system("rm -f $auto_cache_file");
    #}
};


##############################################################################
sub shell {
    my ($package) = \%{$_};

    my ($path, $name, $commit) =
	($package->{'path'},
	 $package->{'name'},
	 $package->{'commit'});

    $commit =~ s/^origin\///;
    $name = "$name-$commit" if ($commit ne "");
    x_chdir("$xorg_git_build_dir/$path/$name");
    x_system($arguments);
};


##############################################################################
# Rpm related functions
my %rpm_build_requires;
my %rpm_trans_table;

# Converts output of xorg-deps.pl that is the full path of a module to
# a package name.
sub rpm_trans {
    my ($module) = @_;

    unless ($rpm_trans_table{$module}) {
	foreach (@modules) {
	    my $commit = $_->{'commit'};
	    if ($commit =~ /^origin/) {
		$commit =~ s/^origin\///;
		$commit = "-$commit";
	    }
	    if (($_->{'path'} ne "" and $module eq "$_->{'path'}/$_->{'name'}$commit")
		or "$_->{'name'}$commit" eq $module) {
		$rpm_trans_table{$module} = $_->{'package'};
		last;
	    }
	}
    }

    return $rpm_trans_table{$module};
};

# Builds name translation table and list build dependencies. This assumes
# a "non rpm build" has been already done.
sub rpm_init {
    my $module;
    my $xorg_deps_pl = `which xorg-deps.pl`;

    chomp($xorg_deps_pl);
    $xorg_deps_pl = "./xorg-deps.pl" unless (-f $xorg_deps_pl);
    die ("$xorg_deps_pl: ($!)") unless (-f $xorg_deps_pl);
    $xorg_deps_pl = "/usr/bin/perl $xorg_deps_pl -p";

    open(DEPS, "$xorg_deps_pl |");
    while (<DEPS>) {
	if (s/^--> //) {
	    chomp;
	    $module = rpm_trans($_);
	    print(STDERR " \*\* Cannot resolve $_\n") if ($module eq undef);
	}
	elsif ($module ne undef and s/BuildRequires: //) {
	    chomp();
	    $_ = rpm_trans($_);
	    push(@{$rpm_build_requires{$module}}, $_)
		if ($_ ne $module);
	}
    }
    close(DEPS);
};

##############################################################################
# Checks if some package needs to be rebuilt, and does it
sub rpm {
    my ($module) = \%{$_};

    my ($path, $name, $configure, $package, $git_commit) =
	($module->{'path'},
	 $module->{'name'},
	 $module->{'configure'},
	 $module->{'package'},
	 $module->{'commit'});

    my $cvs = undef;
    my $SILENT = "";
    my $release = 1;
    my $summary = $cvs ?
	"cvs -d:pserver:anoncvs\@cvs.freedesktop.org:/cvs/$name" :
	"git clone git://anongit.freedesktop.org/git/$path/$name";

    my $rpmname = $package;
    $package .= $package_suffix;

    my ($svnadd, $tmp, $old_version, $old_commit, $version, $commit);

    unless (-f "$xorg_git_rpm_dir/$package/SPECS/$package.spec") {
	x_chdir($xorg_git_rpm_dir);
	if (! -d $package) {
	    x_system("svn co svn+ssh://pcpa\@svn.mandriva.com/svn/packages/cooker/$package/current $package");
	    if (! -d $package) {
		# Should be execute only once...
		x_system("svn mkdir -m '$package' svn+ssh://pcpa\@svn.mandriva.com/svn/packages/cooker/$package");
		x_system("svn mkdir -m '$package' svn+ssh://pcpa\@svn.mandriva.com/svn/packages/cooker/$package/current");
		x_system("svn co svn+ssh://pcpa\@svn.mandriva.com/svn/packages/cooker/$package/current $package");
		x_chdir($package);
		x_system("svn mkdir SPECS");
		x_system("svn mkdir SOURCES");
		$svnadd = 1;
	    }
	}
    }

    if (-f "$xorg_git_rpm_dir/$package/SPECS/$package.spec") {
	$tmp = `head -1 $xorg_git_rpm_dir/$package/SPECS/$package.spec`;
	$tmp =~ m/#([^:]+):(\S+)/;
	$old_version = $1;
	$old_commit = $2;
	$svnadd = 0;
	# Avoid repeated messages on rpm Changelong due to
	# modifying first line.
	## Usual method is just leave it as # only.
	if ($old_version eq "" or $old_commit eq "") {
	    $SILENT = "SILENT: ";
	}
    }
    elsif (! -d "$xorg_git_rpm_dir/$package") {
	die("Cannot create/checkout $package.\n");
    }

    x_chdir("$xorg_git_checkout_dir/$path/$name");

    if ($cvs) {
    	$version = 0;
    	open(LOG, "cvs log |");
    	while (<LOG>) {
    	    if (m/date:\s+(\d+-\d+-\d+)/) {
    		$_ = $1;
    		s/-//g;
    		# Not checking utc
    		if ($_+0 > $version) {
		    $version = $_+0;
    		}
    	    }
    	}
    	$commit = $version;
    	close(LOG);
    }
    else {
	$tmp = $/;
	$/ = undef;
	# Was using head -3, but that causes problems with Merges in last commit
	my $data = `git-log --date=short | head -4`;
	$/ = $tmp;

	$data =~ m/commit\s(\S+)/;
	$commit = $1;
	$data =~ m/Date:\s+(\d+-\d+-\d+)/;
	$version = $1;
	$version =~ s/-//g;
    }

    if ($version <= $old_version or ($commit and $commit eq $old_commit)) {
	# Check build requires, as if build requires changed, this
	# package could still require a rebuild.
	return unless (defined($options{'f'}));

	# Commit may have been pushed late...
	if ($commit ne $old_commit and $version < $old_version) {
	    print(STDERR "using today's date as commit seems to be pushed late.\n");
	    $version = `date +%Y%m%d`;
	    chomp($version);
	}

	$release = `head -4 $xorg_git_rpm_dir/$package/SPECS/$package.spec | tail -1 | sed -e 's/Release: %mkrel //'`;
	$release = $release+0 + 1 if ($options{'f'});
    }

    if ($cvs) {
	x_chdir("..");
	x_system("cp -far $name $package");
	x_system("tar jcf $package.tar.bz2 $package");
	x_system("rm -fr $package");
	x_system("mv $package.tar.bz2 $name");
    }
    else {
	x_system("git-archive --format=tar --prefix=$package/ " . ($git_commit ? $git_commit : "master") . " | bzip2 > $package.tar.bz2");
    }

    x_chdir("$xorg_git_rpm_dir/$package");

    open(SPEC, ">SPECS/$package.spec") or die("Cannot create $package.spec\n");

    print(SPEC "#$version:$commit\n");
    print(SPEC "%define _requires_exceptions lib\n");
    print(SPEC "Name: $package\n");
    print(SPEC "Version: $version\n");
    print(SPEC "Release: %mkrel $release\n");
    print(SPEC "Summary: $summary\n");
    print(SPEC "Group: System/X11\n");
    print(SPEC "URL: http://xorg.freedesktop.org\n");
    print(SPEC "Source: $package.tar.bz2\n");
    print(SPEC "License: MIT\n");
    print(SPEC "BuildRoot: %{_tmppath}/%{name}-root\n\n");
    print(SPEC "BuildConflicts: x11-server-devel\n");
    print(SPEC "BuildConflicts: x11-proto-devel\n");
    print(SPEC "BuildConflicts: libdrm-devel\n");
    print(SPEC "BuildConflicts: GL-devel\n");
    print(SPEC "BuildConflicts: libxcb-devel\n");
    print(SPEC "BuildConflicts: libpciaccess-devel\n");
    print(SPEC "BuildConflicts: libice-devel\n");
    print(SPEC "BuildConflicts: libxau-devel\n");
    print(SPEC "BuildConflicts: libxdmcp-devel\n\n");

    ## XXX Hopefully temporary kludge: x86_64 doesn't accept contrib/testing
    ## packages as dependencies anymore...
    #if ($name =~ m/xf86-video-(amd|geode)/) {
	print(SPEC "ExclusiveArch: %{ix86}\n");
    #}

    $rpmname =~ /(branch.*|X11R7.4)/;
    if ($branches{$1}) {
	my $suffix = $1;
	my @packages = @{$branches{$suffix}{'packages'}};
	foreach my $requires (@{$rpm_build_requires{$rpmname}}) {
	    # Don't build a fontconfig snapshot package anymore on Mandriva
	    next if ($requires eq "fontconfig");

	    if (grep($_ eq $requires, @packages)) {
		$requires .= "-$suffix";
	    }
	    print(SPEC "BuildRequires: $requires$package_suffix\n");
	}
    }
    else {
	foreach (@{$rpm_build_requires{$rpmname}}) {
	    # Don't build a fontconfig snapshot package anymore on Mandriva
	    next if ($_ eq "fontconfig");
	    print(SPEC "BuildRequires: $_$package_suffix\n");
	}
    }
    foreach (@{$module->{'build'}}) {
	print(SPEC "BuildRequires: $_\n");
    }
    foreach (@{$module->{'requires'}}) {
	print(SPEC "Requires: $_\n");
    }

    # Check for those that aren't "parallel" building
    # This should be revied from time to time...
    my $make = "make";
    $make = "\%$make" unless ($name =~ m/^(libXtst|xorg-docs|mesa)$/);
    my $makeinstall = "%makeinstall_std";

    if ($package =~ /^font-/) {
	$makeinstall = "$makeinstall\nrm -f \`find %{buildroot} -type f -name fonts.dir -o -name fonts.scale\`\n";
    }

    print(SPEC "\n%description\n$package is a snapshot of the code from\n");
    print(SPEC "\"$summary $path" . ($path ne "" ? "/" : "") . "$name\"\n");
    print(SPEC "\n%prep\n%setup -q -n $package\n\n%build\n");
    print(SPEC "\nPATH=$package_destdir/bin:\$PATH\n");
    print(SPEC "PKG_CONFIG_PATH=$package_destdir/$xorg_git_lib_name/pkgconfig:\$PKG_CONFIG_PATH\n");
    print(SPEC "LD_LIBRARY_PATH=$package_destdir/$xorg_git_lib_name:\$LD_LIBRARY_PATH\n");
    if ($name ne "macros") {
	print(SPEC "ACLOCAL_LOCALDIR=$package_destdir/share/aclocal\n");
	print(SPEC "ACLOCAL=\"aclocal -I \$ACLOCAL_LOCALDIR\"\n");
	print(SPEC "export PATH PKG_CONFIG_PATH LD_LIBRARY_PATH ACLOCAL_LOCALDIR ACLOCAL\n");
    }
    else {
	print(SPEC "export PATH PKG_CONFIG_PATH LD_LIBRARY_PATH");
    }
    if ($name eq "xserver") {
	print(SPEC "CFLAGS=\"-DDEFAULT_CONF_PATH=\\\\\"\\\"$package_destdir/etc/X11/xorg.conf\\\\\"\\\" -DROOT_CONFIGPATH=DEFAULT_CONF_PATH -DUSER_CONFIGPATH=DEFAULT_CONF_PATH\"\n");
	print(SPEC "export CFLAGS\n");
    }

    if ($name eq "mesa") {
	if ($git_commit) {
	    $make= "make "				.
	    "OPT_FLAGS=\"-I$package_destdir/include $ENV{CFLAGS}\" ".
	    "X11_INCLUDES=-I$package_destdir/include "	.
	    "EXTRA_LIB_PATH=-L$package_destdir/lib "	.
	    "LIB_DIR=lib "				.
	    "INCLUDE_DIR=$package_destdir/include "	.
	    "DRI_DRIVER_DIR=$package_destdir/lib/dri "	.
	    "MKDEP=$package_destdir/bin/gccmakedep "	.
	    "linux-dri";
	    $makeinstall = "make "			.
	    "OPT_FLAGS=\"-I$package_destdir/include $ENV{CFLAGS}\" ".
	    "X11_INCLUDES=-I$package_destdir/include "	.
	    "EXTRA_LIB_PATH=-L$package_destdir/lib "	.
	    "INSTALL_DIR=$package_destdir "		.
	    "DRI_DRIVER_INSTALL_DIR=$final_destdir/lib/dri " .
	    "install";
	}
	else {
	    print(SPEC "\nEXTRA_LIB_PATH=-L$package_destdir/$xorg_git_lib_name\n");
	    print(SPEC "export EXTRA_LIB_PATH\nmake configure\n");
	    print(SPEC "./configure --prefix=$package_destdir $ac_libdir $configure");
	}
    }
    else {
	print(SPEC "\n./autogen.sh --prefix=$package_destdir $ac_libdir $configure");
    }

    print(SPEC "\n\n$make\n\n%install\nrm -fr %{buildroot}\n$makeinstall\n");

    if ($name eq "mesa") {
	print(SPEC "\n%ifnarch %{sparc}\nrm -f %{buildroot}/$package_destdir/lib/dri/ffb_dri.so\n%endif\n");
    }
    elsif ($name eq "drm") {

	print(SPEC "DRMDIR=%{_usrsrc}/dkms-%{name}\n");
	print(SPEC "mkdir -p %{buildroot}\$DRMDIR\n");
	print(SPEC "ln -s \$DRMDIR %{buildroot}\$DRMDIR-%{version}\n");
	print(SPEC "cp -favr shared-core linux-core scripts %{buildroot}\$DRMDIR\n");
	print(SPEC "cat > %{buildroot}\$DRMDIR/dkms.conf <<EOF\n");
	print(SPEC "PACKAGE_NAME=\"dkms-%{name}\"\n");
	print(SPEC "PACKAGE_VERSION=\"%{version}\"\n");
	print(SPEC "MAKE[0]=\"make -C linux-core LINUXDIR=\\\$kernel_source_dir\"\n");
	print(SPEC "CLEAN=\"make -C linux-core clean\"\n");
	print(SPEC "AUTOINSTALL=YES\n");
	my $i = 0;
	foreach my $drm_module (@drm_modules) {
	    next if ($git_commit eq "origin/drm-gem" && $drm_module eq "i810");
	    print(SPEC "BUILT_MODULE_NAME[$i]=\"$drm_module\"\n");
	    print(SPEC "BUILT_MODULE_LOCATION[$i]=\"linux-core\"\n");
	    print(SPEC "DEST_MODULE_LOCATION[$i]=\"/kernel/drivers/char/drm\"\n");
	    $i++;
	}
	print(SPEC "EOF\n");

	# Avoid dkms build failure due to not being built from a git
	# repository and/or git not available.
	print(SPEC
	      "perl -pi -e 's|^(GIT_REVISION :=)|\$1 ",
	      `(cd $xorg_git_checkout_dir/$path/$name && echo -n \`git-describe --abbrev=17\`)`,
	      "|' %{buildroot}\$DRMDIR/linux-core/Makefile");
    }

    print(SPEC "\n%clean\nrm -rf %{buildroot}\n");
    print(SPEC "\n%files\n%defattr(-,root,root)\n");

    if ($name eq "libXaw") {
	print(SPEC "$package_destdir/lib/libXaw.so*\n");
    }


    # Replace "from source destdir" with package destdir, and leave a
    # place to allow adding extra hooks on filename/path transformation.
    if ($git_commit) {
	$git_commit =~ s/^origin\///;
	$git_commit = "-$git_commit";
    }
    open(FILES, "$xorg_git_deps_dir/$path/$name$git_commit.files")
	or die ("Cannot read $xorg_git_deps_dir/$path/$name$git_commit.files\n");
    while (<FILES>) {
	s/^$xorg_git_dest_dir/$package_destdir/;
	# It executes symlink("old", "new") using relative pathnames,
	# and tracking the current dir in strace output isn't that easy...
	s/lib([a-zA-Z0-9-]+)\.so\.\d+\.\d+(\.\d+)?$/lib$1.so\*/;
	# Manpages are compressed
	s/(share\/man\/man\d\/.*\.\d)$/$1\*/;

	# xorg-docs compress ps
	s/(share\/X11\/doc\/.*\.ps)$/$1\*/;

	# fonts.scale and fonts.dir are dynammically generated
	next if (m/\/fonts\.(scale|dir)$/);

	if ($name eq "mesa" and
	    m/dri\/(i810|s3v|sis|trident)_dri\.so/) {
	    print(SPEC "%ifarch %{ix86}\n$_%endif\n");
	}
	elsif ($name eq "mesa" and
	    m/dri\/ffb_dri\.so/) {
	    print(SPEC "%ifarch %{sparc}\n$_%endif\n");
	}
	elsif ($name eq "xserver") {
	    if (m/bin\/X(ati|chips|i810|mach64|mga|nvidia|pm2|r128|smi|vesa|via)?$/) {
		print(SPEC "%ifarch %{ix86} alpha\n$_%endif\n");
	    }
	    # XXX Install setuid XServer
	    elsif (m/^($package_destdir\/bin\/Xorg)$/) {
		print(SPEC "%attr(4755, root, root) $_");
	    }
	    else {
		print(SPEC $_);
	    }
	}
	elsif ($name eq "xkeyboard-config" and
	       m/rules\/(sgi|sun)(.xml)?/) {
	    # say hi ... Fixme
	}
	else {
	    print(SPEC $_);
	}
    }
    close(FILES);
    ## Symlinks should really be handled in xorg-trace.pl...
    if ($name eq "xserver") {
	print(SPEC "$package_destdir/bin/X\n");
    }
    elsif ($name eq "xf86-video-intel") {
	print(SPEC "$package_destdir/lib/xorg/modules/drivers/i810_drv.so\n");
    }
    elsif ($name eq "xkeyboard-config") {
	print(SPEC "$package_destdir/share/X11/xkb/rules/xorg\n");
	print(SPEC "$package_destdir/share/X11/xkb/rules/xorg.lst\n");
	print(SPEC "$package_destdir/share/X11/xkb/rules/xorg.xml\n");
    }
    elsif ($name eq "drm") {
	print(SPEC "\n%package -n dkms-%{name}\nSummary: Kernel drm modules\n");
	print(SPEC "Group: Development/C\n\n");
	print(SPEC "%description -n dkms-%{name}\n");
	print(SPEC "Kernel drm modules.\n\n");
	print(SPEC "These modules are the latest development version.\n\n");
	print(SPEC "Requires: dkms\n");

	print(SPEC "Obsoletes: %{name}-dkms\n")	if ($rpmname eq "libdrm");

	print(SPEC "Requires(post): dkms\n");
	print(SPEC "Requires(preun): dkms\n");

	print(SPEC "\n%post -n dkms-%{name}\n");
	print(SPEC "dkms add     -m dkms-%{name} -v %{version} --rpm_safe_upgrade &&\n");
	print(SPEC "dkms build   -m dkms-%{name} -v %{version} --rpm_safe_upgrade &&\n");
	print(SPEC "dkms install -m dkms-%{name} -v %{version} --rpm_safe_upgrade --force\n");
	print(SPEC "rmmod ", join(" ", @drm_modules), " nvidia fglrx &>/dev/null\n");
	print(SPEC "true\n");
	print(SPEC "\n%preun -n dkms-%{name}\n");
	print(SPEC "dkms remove  -m dkms-%{name} -v %{version} --rpm_safe_upgrade --all\n");
	print(SPEC "rmmod ", join(" ", @drm_modules), " nvidia fglrx &>/dev/null\n");
	print(SPEC "true\n");

	print(SPEC "\n%files -n dkms-%{name}\n%defattr(-,root,root)\n");
	print(SPEC "%{_usrsrc}/dkms-%{name}-%{version}\n");
	print(SPEC "%{_usrsrc}/dkms-%{name}/*\n");
    }

    close(SPEC);

    x_system("mv -f $xorg_git_checkout_dir/$path/$name/*.tar.bz2 $xorg_git_rpm_dir/$package/SOURCES");

    x_system("svn add SPECS/*.spec SOURCES/*.bz2") if ($svnadd == 1);
    x_system("svn ci -m '$SILENT$rpmname $version snapshot'");
    my $commit = `svn update`;
    $commit =~ s/\D//g;
    x_system("repsys submit -t cooker --define section=contrib/testing svn+ssh://svn.mandriva.com/svn/packages/cooker/$package $commit", 1);
};


##############################################################################
# Replace hash values with macros in the format %{string} to $module->{'string'}
#   Only one pass and no specific order
sub replace_macros_and_check_errors {
    my ($module, $key, $value, $macro, $replace);

    foreach $module (@modules) {
	while (($key, $value) = each %$module) {
	    # Don't allow modules without a version specified
	    #die("$module->{package} doesn't have a version")
		#unless($module->{'version'} or $module->{'skip'});

	    # Hack to allow escaping macros in automatic generation of rpm specs
	    $value =~ s/%%/<%>/g;

	    while ($value =~ /%\{(\w+)\}/) {
		$macro = $1;
		if ($module->{$macro}) {
		    $replace = $module->{$1};
		}
		else {
		    # Don't allow undefined ``macros''.
		    # Assume name is always properly defined
		    die(" ** unknown macro $module->{name}:%{$macro}.\n");
		}
		$value =~ s/%\{$1\}/$replace/g;
	    }

	    $value =~ s/<%>/%/g;
	    $module->{$key} = $value;
	}
    }
}
replace_macros_and_check_errors();


##############################################################################
# Subroutines are free to change this now as a lot of "system" commands
# may be used.
my $dir = Cwd::getcwd();

my %table = (
    "update"	=>	\&update,
    "checkout"	=>	\&checkout,
    "build"	=>	\&build,
    "checktars"	=>	\&checktars,
    "shell"	=>	\&shell,
    "rpm"	=>	\&rpm,
);

#  Add skip for -n option
if ($options{'n'}) {
    foreach (@modules) {
	if ($options{'n'} =~ m/\//) {
	    $_->{'skip'} = "yes"
	        unless ($_->{'path'}			.
			($_->{'path'} ne "" ? "/" : "")	.
			$_->{'name'} eq $options{'n'});
	}
	else {
	    $_->{'skip'} = "yes" unless ($_->{'name'} eq $options{'n'});
	}
    }
}
elsif ($options{'p'}) {
    foreach (@modules) {
	$_->{'skip'} = "yes" unless ($_->{'package'} eq $options{'p'});
    }
}
# Skip any packages with branches by default
else {
    foreach (@modules) {
	$_->{'skip'} = "yes" if (defined($_->{'commit'}));
    }
}

#  Add skip for modules without a 'package' entry
if ($rpm) {
    rpm_init() if ($command eq "rpm");
    foreach (@modules) {
	$_->{'skip'} = "yes" unless ($_->{'package'});
    }
}

# If using a temp file to not need to restart over an update...
if ($command eq "update" and not $options{'c'} and -f $git_last_update) {
    my $lastupdate = `cat $git_last_update`;
    foreach (@modules) {
	last if ("$_->{path}/$_->{name}" eq $lastupdate);
	$_->{'skip'} = "yes";
    }
}
foreach (@modules) {
    print(STDERR "--> $_->{name} $_{commit}\n") unless ($options{'n'});
    my $function = $table{$command};
    $function->($_) unless ($_->{'skip'});
}
# If using a temp file to not need to restart over an update...
if ($command eq "update" and -f $git_last_update) {
    x_system("rm -f $git_last_update");
}

# Restore current directory
chdir($dir);
